//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Version 2.4		$Date: 2006/11/13 09:08:28 $
//
// Category     : VST 2.x SDK Samples
// Filename     : editor.cpp
// Created by   : Steinberg Media Technologies
// Description  : plugin tamplatee with Editor using VSTGUI
//
//  2006, Steinberg Media Technologies, All Rights Reserved
//
// Audified Plug-In Template
// Version 1.1		$Date: 2007/11/22 $
// Created by   : Audified Solutions
// Description  : Audified DPlug-In Template
//
//  2007, Audified Solutions, DISK Multimedia, s.r.o.
//-------------------------------------------------------------------------------------------------------

#ifndef __vsttemp__
#include "../source/vst_temp.h"
#endif

#ifndef __editor__
#include "editor.h"
#endif

#ifdef def_Editor

#include <stdio.h>

//-----------------------------------------------------------------------------
// resource id's
enum {
	// bitmaps
	kBackgroundId = 128,

	kFaderBodyId,
	kFaderHandleId,
	kScaleBodyId,
	kScaleHandleId,
	
	kLabelWidth = 40,
	kParamWidth = 30,
	kDisplayWidth = 80,
	kFrameWidth = 360,

	kYTail = 10,
	kYHead = kYTail + 30,
	kXOffset = 10,
	kXPad = 10,
	kYPad = 2,
	kYStep = 16,

};

//-----------------------------------------------------------------------------
CEditor::CEditor (AudioEffect *effect) : AEffGUIEditor (effect) 
{
	short nRows = PARAM_NUMBER;
#ifdef def_RowsInEditor
	nRows = def_RowsInEditor;
#endif
	short nColumns = short((float)(PARAM_NUMBER-1)/(float)nRows)+1;

	rect.left   = 0;
	rect.top    = 0;
	rect.right  = kFrameWidth * nColumns;
	rect.bottom = (short)(nRows * kYStep + kYHead + kYTail);
	hBackground = 0;

	for (VstInt32 i = 0; i < PARAM_NUMBER; i++) {
		myFader[i] = 0;
		myDisplay[i] = 0;
		myLabel[i]=0;
		myParam[i]=0;
	}
}

//-----------------------------------------------------------------------------
CEditor::~CEditor ()
{
	if (hBackground)
		hBackground->forget ();
	hBackground = 0;
}

//-----------------------------------------------------------------------------
bool CEditor::open (void *ptr)
{
	AEffGUIEditor::open (ptr);

	short nRows = PARAM_NUMBER;
#ifdef def_RowsInEditor
	nRows = def_RowsInEditor;
#endif
	short nColumns = short((float)(PARAM_NUMBER-1)/(float)nRows)+1;

	CRect size (0, 0, kFrameWidth * nColumns, nRows * kYStep + kYHead + kYTail);
	CFrame* lFrame = new CFrame (size, ptr, this);
	lFrame->setTransparency(false);
	lFrame->setBackgroundColor(kBlackCColor);

	CBitmap* hFaderBody   = new CBitmap (kFaderBodyId);
	CBitmap* hFaderHandle = new CBitmap (kFaderHandleId);
	CBitmap* hScaleBody   = new CBitmap (kScaleBodyId);
	CBitmap* hScaleHandle = new CBitmap (kScaleHandleId);
	char text[kVstMaxParamStrLen+1];

	short kXpos = kXOffset;
	short kHeight = (short)hFaderBody->getHeight();

	// CTextLabel
	for (VstInt32 i = 0; i < PARAM_NUMBER; i++) 
	{
		short kXpos2 = kXpos + short((float)i/(float)nRows) * kFrameWidth;
		size (kXpos2, kYHead + (i%nRows)*kYStep, kXpos2 + kLabelWidth, kYHead + (i%nRows)*kYStep + kHeight);
		myLabel[i] = new CTextLabel (size, 0, 0, 0);
		myLabel[i]->setHoriAlign (kLeftText);
		myLabel[i]->setFont (kNormalFontSmall);
		if( ((CVSTTemplate*)effect)->isParameterReadOnly(i) ) 
			myLabel[i]->setFontColor (kGreenCColor);
		else
			myLabel[i]->setFontColor (kYellowCColor);
		myLabel[i]->setBackColor (kBlackCColor);
		myLabel[i]->setFrameColor (kBlackCColor);
		effect->getParameterName (i, text);
		myLabel[i]->setText (text);
		lFrame->addView (myLabel[i]);
	}
	kXpos += kLabelWidth + kXPad;

	// CHorizontalSlider
	int minPos = kXpos;
	int maxPos = kXpos + hFaderBody->getWidth () - hFaderHandle->getWidth () - 1;
	CPoint point (0, 0);
	CPoint offset (0, 1);
	for (VstInt32 i = 0; i < PARAM_NUMBER; i++) 
	{
		short kXpos2 = kXpos + short((float)i/(float)nRows) * kFrameWidth;
		size (kXpos2, kYHead + (i%nRows)*kYStep, kXpos2 + hFaderBody->getWidth (), kYHead + (i%nRows)*kYStep + hFaderBody->getHeight ());
		if( ((CVSTTemplate*)effect)->isParameterReadOnly(i) ) 
			myFader[i] = new CHorizontalSlider (size, this, i, minPos + short((float)i/(float)nRows) * kFrameWidth, maxPos + short((float)i/(float)nRows) * kFrameWidth, hScaleHandle, hScaleBody, point, kLeft);
		else
			myFader[i] = new CHorizontalSlider (size, this, i, minPos + short((float)i/(float)nRows) * kFrameWidth, maxPos + short((float)i/(float)nRows) * kFrameWidth, hFaderHandle, hFaderBody, point, kLeft);
		myFader[i]->setOffsetHandle (offset);
		myFader[i]->setValue (effect->getParameter (i));
		if( ((CVSTTemplate*)effect)->isParameterReadOnly(i) ) 
			myFader[i]->setMouseEnabled(false);
		lFrame->addView (myFader[i]);
	}
	kXpos += (short)hFaderBody->getWidth() + kXPad;

	// CParamDisplay
	for (VstInt32 i = 0; i< PARAM_NUMBER; i++) 
	{
		short kXpos2 = kXpos + short((float)i/(float)nRows) * kFrameWidth;
		size (kXpos2, kYHead + (i%nRows)*kYStep,  kXpos2 + kParamWidth, kYHead + (i%nRows)*kYStep + kHeight);
		myParam[i] = new CParamDisplay (size, 0, kCenterText);
		myParam[i]->setFont (kNormalFontSmall);
		myParam[i]->setBackColor (kBlackCColor);
		myParam[i]->setFrameColor (kGreyCColor);
		myParam[i]->setFontColor (kGreyCColor);
		myParam[i]->setValue (effect->getParameter (i));
		lFrame->addView (myParam[i]);
	}
	kXpos += kParamWidth + kXPad;

	// CTextLabel
	char caption[2*kVstMaxParamStrLen+2];
	for (VstInt32 i = 0; i< PARAM_NUMBER; i++) 
	{
		short kXpos2 = kXpos + short((float)i/(float)nRows) * kFrameWidth;
		size (kXpos2, kYHead + (i%nRows)*kYStep, kXpos2 + kDisplayWidth, kYHead + (i%nRows)*kYStep + kHeight);
		myDisplay[i] = new CTextLabel (size, 0, 0, kRightText);
		myDisplay[i]->setHoriAlign (kRightText);
		myDisplay[i]->setFont (kNormalFontSmall);
		if( ((CVSTTemplate*)effect)->isParameterReadOnly(i) ) 
			myDisplay[i]->setFontColor (kGreenCColor);
		else
			myDisplay[i]->setFontColor (kWhiteCColor);
		myDisplay[i]->setBackColor (kBlackCColor);
		myDisplay[i]->setFrameColor (kBlackCColor);
		effect->getParameterDisplay (i, caption);
		effect->getParameterLabel (i, text);
		strcat(caption," ");
		myDisplay[i]->setText (strcat(caption,text));
		lFrame->addView (myDisplay[i]);
	}
	kXpos += kDisplayWidth + kXPad;

	hFaderBody->forget ();
	hFaderHandle->forget ();
	hScaleBody->forget ();
	hScaleHandle->forget ();
	frame = lFrame;
	return true;
}

//-----------------------------------------------------------------------------
void CEditor::close ()
{
	delete frame;
	frame = 0;
}

//-----------------------------------------------------------------------------

void CEditor::setParameter (VstInt32 index, float value)
{
	if (frame == 0)
		return;

	char text[kVstMaxParamStrLen+1];
	char caption[2*kVstMaxParamStrLen+2];

	//value = effect->getParameter (index);

	switch (index)	{
		case 0:
		default:
			if (myFader[index]) {
				myFader[index]->setValue (value);
			}
			if (myDisplay[index]) {
				effect->getParameterDisplay (index, caption);
				effect->getParameterLabel (index, text);
				strcat(caption," ");
				myDisplay[index]->setText (strcat(caption,text));
				myDisplay[index]->setDirty ();
			}
			if (myParam[index]) {
				myParam[index]->setValue (value);
				myParam[index]->setDirty ();
			}
	}
}

//-----------------------------------------------------------------------------
void CEditor::valueChanged (CDrawContext* context, CControl* control)
{
	long tag = control->getTag ();
	switch (tag) {
		case 0:
		default: 
			effect->setParameterAutomated (tag, control->getValue ());
			control->setDirty ();
	}
}

//-----------------------------------------------------------------------------
void CEditor::idle (void)
{
	AEffGUIEditor::idle();

	for (VstInt32 i = 0; i < PARAM_NUMBER; i++ )
		if( ((CVSTTemplate*)effect)->isParameterReadOnly(i) ) {
			setParameter (i, effect->getParameter (i));
		}
}
//-----------------------------------------------------------------------------

#endif
