//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Version 2.4		$Date: 2006/11/13 09:08:27 $
//
// Category     : VST 2.x SDK Samples
// Filename     : vst_temp.h
// Created by   : Steinberg Media Technologies
// Description  : Plugin template
//
//  2006, Steinberg Media Technologies, All Rights Reserved
//
// Plug-In Template
// Version 1.2		$Date: 2008/05/03 $
// Created by   : Jiri Schimmel
// Description  : VST Plug-In Template
//
//  2008, Brno University of Technology
//-------------------------------------------------------------------------------------------------------

#ifndef __vsttemp__
#include "vst_temp.h"
#endif

#ifndef __editor__
#include "editor/editor.h"
#endif


//*******************************************************************************************************
//******************************************     USER CODE     ******************************************
//*******************************************************************************************************

// include specific headers
#include <math.h>
#include "fcie.h"
#include "core.h"
#include "matica.h"

// default program name (max 24 chars)
const char def_DefaultProgName[kVstMaxProgNameLen] = "Default";
// vendor text identifier (max 64 chars)
const char def_VendorString[kVstMaxVendorStrLen]   = "Brno University of Technology";
// product text identifier (max 64 chars)
const char def_ProductString[kVstMaxProductStrLen] = "VST plug-in Template";
// effect name (max 32 chars)
const char def_EffectName[kVstMaxEffectNameLen]    = "VST template";

// user-defined constants
const float pi = 3.14159265358979f;

// real parameters definition
ParamInfo params[PARAM_NUMBER] = {};
ParamInfo unsortedParams[PARAM_NUMBER] =
{
// tag, current, default, min, max, name, label, show, read only
	{ kGain,   0.0f, 0.0f, 0.0f, 1.0f, "DIST", "", SHOW_FLOAT, false },
	{ kFreq,   0.0f, 0.0f, 0.0f, 1.0f, "TONE", "", SHOW_FLOAT, false },
	{ kQual,   0.0f, 0.0f, 0.0f, 1.0f, "VOL", "", SHOW_FLOAT, false },
	{ kBypass, 0.0f, 1.0f, 0.0f, 1.0f, "Bypass", "", SHOW_ONOFF, false },
};

//-----------------------------------------------------------------------------------------
// initialization of parameters and user variables defined in user_variables.h
void CVSTTemplate::setUserVariables (void)
{
	for (int i = 0; i < def_ChNumber;i++)
	{
		d[i] = new core(3,getSampleRate(),var3,num3);
		d2[i] = new core(2,getSampleRate(),var2,num2);
		mem[i] = (float*)calloc(2,sizeof(float));
		mem0[i] = (float*)calloc(2,sizeof(float));
		mem1[i] = (float*)calloc(2,sizeof(float));
		mem2[i] = (float*)calloc(2,sizeof(float));
	}
}

//-----------------------------------------------------------------------------------------
// deinitialization of dynamic user variables created in setUserVariables()
void CVSTTemplate::clearUserVariables (void)
{
	for (int i = 0; i < def_ChNumber;i++)
	{
		if(d[i])
			delete d[i];
		if(d2[i])
			delete d2[i];
		if(mem[i])
			delete mem[i];
		if(mem0[i])
			delete mem0[i];
		if(mem1[i])
			delete mem1[i];
		if(mem2[i])
			delete mem2[i];
	}
}

//-----------------------------------------------------------------------------------------
bool CVSTTemplate::parameterChanged (VstInt32 index, float value)
{

	float m;
	const float pi = 3.1415;
	float V, T, G, V0, den, a[3], b[3], x, c;

	switch (index)
	{
	case kBypass:
	case kGain:
	case kFreq:
	case kQual:
	default:
		{
			G = .7*getRealValue(kGain);
			T = getRealValue(kFreq);
			V = log10((9*(1-getRealValue(kQual)))+1);
			c = 2.*getSampleRate();
			a00=1., a01=-1.9983, a02=0.9983, b00=0.99377, b01=-1.9875, b02=0.9937;
			//a30=1., a31=-1.9983, a32=0.9983, b30=0.99377, b31=-1.9875, b32=0.9937;
			a0=1., a1=-1.9991, a2=0.9991, b0=0.9713, b1=-1.9427, b2=0.9713;
		
			b[0] = 1;
			b[1] = 4700*1e-6 +(100e3-(100e3*G))*1e-6 +(100e3*G)*250e-12 +(100e3*G)*1e-6;
			b[2] = (100e3*G)*250e-12*4700*1e-6 + (100e3*G)*(100e3-(100e3*G))*250e-12*1e-6;
			a[0] = 1;
			a[1] = 4700*1e-6 +(100e3-(100e3*G))*1e-6 +(100e3*G)*250e-12;
			a[2] = (100e3*G)*250e-12*4700*1e-6 +(100e3*G)*(100e3-(100e3*G))*250e-12*1e-6;

			a10 = a[0] + a[1]*c + a[2]*c*c;
			x = 1/a10;
			a10 = x*a10;
			a11 = x*(2.*a[0] - 2.*a[2]*c*c);
			a12 = x*(a[0] - a[1]*c + a[2]*c*c);
			b10 = x*(b[0] + b[1]*c + b[2]*c*c);
			b11 = x*(2.*b[0] - 2.*b[2]*c*c);
			b12 = x*(b[0] - b[1]*c + b[2]*c*c);

			
			/*R16 = 6800;
			C11 = 22e-9;
			R15	= 2200;
			R17 = 6800;
			C12 = 100e-9;
			RP1 = in1;					// 20e3*T
			RP2 = in2;					// 20e3-(20e3*T)
			RV1 = in11;					// 100e3-(100e3*V)
			RV2 = in22;					// 100e3*V
*/
			b[0] = (20e3-(T*20e3))*(100e3-(99999*V)) +6800*(100e3-(99999*V));
			b[1] = ( 6800*22e-9*(20e3*T)*(100e3-(99999*V)) +6800*22e-9*(20e3-(20e3*T))*(100e3-(99999*V)) 
				+2200*22e-9*(20e3-(20e3-T))*(100e3-(99999*V)) +2200*6800*22e-9*(100e3-(99999*V)) +6800*6800*22e-9*(100e3-(99999*V)) );
			b[2] = ( 6800*6800*22e-9*100e-9*(20e3*T)*(100e3-(99999*V)) );
			//%--------------------------------------
			a[0] = (20e3-(20e3*T))*(100e3-(99999*V)) +(20e3-(20e3*T))*(100e3*V) +(20e3*T)*(100e3-(99999*V)) 
				+(20e3*T)*(100e3*V) +(20e3*T)*(20e3-(20e3*T)) +6800*(100e3*V) +6800*(20e3*T) +6800*(100e3-(99999*V))
				+6800*6800 +6800*(20e3-(20e3*T)) +6800*(100e3-(99999*V)) +6800*(100e3*V);
			a[1] =  6800*22e-9*(20e3-(20e3*T))*(100e3-(99999*V)) +6800*22e-9*(20e3*T)*(100e3-(99999*V)) 
				+6800*22e-9*(20e3*T)*(100e3*V) +6800*22e-9*(20e3-(20e3*T))*(100e3*V) +2200*22e-9*(20e3*T)*(100e3*V) 
				+2200*22e-9*(20e3*T)*(20e3-(20e3*T)) +2200*6800*22e-9*(100e3-(99999*V)) +2200*6800*22e-9*(100e3*V) 
				+2200*6800*22e-9*(20e3*T) +6800*22e-9*(20e3*T)*(20e3-(20e3*T)) +6800*100e-9*(20e3-(20e3*T))*(100e3*V) 
				+6800*100e-9*(20e3*T)*(100e3-(99999*V)) +6800*100e-9*(20e3*T)*(100e3*V) 
				+6800*100e-9*(20e3*T)*(20e3-(20e3*T)) +2200*22e-9*(20e3-(20e3*T))*(100e3-(99999*V)) 
				+6800*100e-9*(20e3-(20e3*T))*(100e3-(99999*V)) +2200*22e-9*(20e3*T)*(100e3-(99999*V)) 
				+6800*6800*22e-9*(100e3*V) +6800*6800*22e-9*(20e3-(20e3*T)) +6800*6800*100e-9*(100e3-(99999*V))	
				+6800*6800*100e-9*(100e3*V) +6800*2200*22e-9*(100e3-(99999*V)) +6800*6800*22e-9*(100e3-(99999*V)) 
				+6800*2200*22e-9*(100e3*V) +6800*2200*22e-9*(20e3-(20e3*T)) +2200*22e-9*(20e3-(20e3*T))*(100e3*V) 
				+6800*6800*100e-9*(20e3*T) +6800*2200*6800*22e-9 ;
			a[2] =  6800*6800*22e-9*100e-9*(20e3*T)*(200e3*V) +6800*6800*22e-9*100e-9*(20e3-(20e3*T))*(100e3-(99999*V))	
				+6800*6800*22e-9*100e-9*(20e3-(20e3*T))*(100e3*V) +6800*6800*22e-9*100e-9*(20e3*T)*(100e3-(99999*V)) 
				+6800*2200*22e-9*100e-9*(20e3*T)*(100e3*V) +6800*2200*22e-9*100e-9*(20e3*T)*(20e3-(20e3*T)) 
				+6800*2200*22e-9*100e-9*(20e3*T)*(100e3-(99999*V)) 
				+6800*2200*22e-9*100e-9*(20e3-(20e3*T))*(100e3-(99999*V))
				+6800*2200*22e-9*100e-9*(20e3-(20e3*T))*(100e3*V) +6800*2200*6800*22e-9*100e-9*(100e3-(99999*V))
				+6800*2200*6800*22e-9*100e-9*(100e3*V) +6800*2200*6800*22e-9*100e-9*(20e3*T) 
				+6800*6800*22e-9*100e-9*(20e3*T)*(20e3-(20e3*T));
			
			
			a20 = a[0] + a[1]*c + a[2]*c*c;
			x = 1/a20;
			a20 = x*a10;
			a21 = x*(2.*a[0] - 2.*a[2]*c*c);
			a22 = x*(a[0] - a[1]*c + a[2]*c*c);
			b20 = x*(b[0] + b[1]*c + b[2]*c*c);
			b21 = x*(2.*b[0] - 2.*b[2]*c*c);
			b22 = x*(b[0] - b[1]*c + b[2]*c*c);


			
			
			/*G = getRealValue(kGain);
			fc = getRealValue(kFreq);
			Q = getRealValue(kQual);

			m = tan(pi*fc/sampleRate);
			if (G>0) {
			V0 = pow(10,(G/20));
			den = 1 + (1/Q)*m + m*m;
			b0 = (1 + (V0/Q)*m + m*m) / den;
			b1 = (2*(m*m - 1)) / den;
			b2 = (1 - (V0/Q)*m + m*m) / den;
			a1 = (2*(m*m - 1)) / den;
			a2 = (1 - (1/Q)*m + m*m) / den;
			}
			else if (G<0) {
			V0 = pow(10,(-G/20));
			den = 1 + (V0/Q)*m + m*m;
			b0 = (1 + (1/Q)*m + m*m) / den;
			b1 = (2*(m*m - 1)) / den;
			b2 = (1 - (1/Q)*m + m*m) / den;
			a1 = (2*(m*m - 1)) / den;
			a2 = (1 - (V0/Q)*m + m*m) / den;
			}
			else {
				b0 = 1;
				b1 = 0;
				b2 = 0;
				a1 = 0;
				a2 = 0;
			}*/
		}


		return true;
	}
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames)
{
// --------------------- do not change this ---------------------
	if(!allocFloatTempBuffers(sampleFrames))
		return;
	float *in, *out, *out2;
	VstInt32 frames;
// --------------------------------------------------------------

	// begin channel loop
	//for (VstInt32 channel = 0; channel < def_ChNumber; channel++ ) 
	//{
		//bypass
	VstInt32 channel = 0;
		if(getRealValue(kBypass))
			memcpy(outputs[channel], inputs[channel], sampleFrames * sizeof(float));
		else
		{
			// init i/o pointers
			in =  getInputBuffer (inputs,  sampleFrames, channel);
			out = getOutputBuffer(outputs, sampleFrames, 0);
			out2 = getOutputBuffer(outputs, sampleFrames, 1);
			frames = sampleFrames;

			// begin sample loop
			//while (--frames >= 0)
			//{
			//	// PLACE YOUR PROCESSING CODE HERE
			//	*out++ = getRealValue(kGain) * (*in++);				

			//} // end sample loop

			float* ch_mem = mem0[channel];
			core* ch_d = d[channel];
			for (int i = 0; i < frames; i++)
			{
				out[i] =    b00*in[i] + ch_mem[0];
				ch_mem[0] = b01*in[i] - a01*out[i] + ch_mem[1];
				ch_mem[1] = b02*in[i] - a02*out[i];
			}
			
			ch_d->clipper(out,in,frames);
			/*for (int i = 0; i < frames; i++)
			{
				in[i] = in[i]*.5;
			}*/
			ch_mem = mem1[channel];
			for (int i = 0; i < frames; i++)
			{
				out[i] =    b10*in[i] + ch_mem[0];
				ch_mem[0] = b11*in[i] - a11*out[i] + ch_mem[1];
				ch_mem[1] = b12*in[i] - a12*out[i];

			}
			ch_d = d2[channel];
			ch_d->clipper(out,in,frames);
			ch_mem = mem2[channel];
			for (int i = 0; i < frames; i++)
			{
				out[i] =    b20*in[i] + ch_mem[0];
				ch_mem[0] = b21*in[i] - a21*out[i] + ch_mem[1];
				ch_mem[1] = b22*in[i] - a22*out[i];

			}
			for(int i=0; i<frames; i++)
				in[i] = out[i];

			/*ch_mem = mem[channel];
			for (int i = 0; i < frames; i++)
			{
				out[i] =    b0*in[i] + ch_mem[0];
				ch_mem[0] = b1*in[i] - a1*out[i] + ch_mem[1];
				ch_mem[1] = b2*in[i] - a2*out[i];

			}*/
			for(int i=0; i<frames; i++)
				out2[i] = out[i];
			

// --------------------- do not change this ---------------------
			flushOutputBuffer (outputs, sampleFrames, 0);
			flushOutputBuffer (outputs, sampleFrames, 1);
// --------------------------------------------------------------

		} // not bypassed
	 //}// end channel loop

// --------------------- do not change this ---------------------
	totalSampleFrames += sampleFrames;
// --------------------------------------------------------------
}

//-----------------------------------------------------------------------------------------
#ifdef def_canDoubleReplacing
void CVSTTemplate::processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames)
{
// --------------------- do not change this ---------------------
	if(!allocDoubleTempBuffers(sampleFrames))
		return;
	double *in, *out;
	VstInt32 frames;
// --------------------------------------------------------------

	// begin channel loop
	for (VstInt32 channel = 0; channel < def_ChNumber; channel++ ) 
	{
		//bypass
		if(getRealValue(kBypass))
			memcpy(outputs[channel], inputs[channel], sampleFrames * sizeof(double));
		else
		{
			// init i/o pointers
			in =  getInputBuffer (inputs,  sampleFrames, channel);
			out = getOutputBuffer(outputs, sampleFrames, channel);
			frames = sampleFrames;

			// begin sample loop
			while (--frames >= 0)
			{
				// PLACE YOUR PROCESSING CODE HERE
				*out++ = (double)getRealValue(kGain) * (*in++);

			} // end sample loop

// --------------------- do not change this ---------------------
			flushOutputBuffer (outputs, sampleFrames, channel);
// --------------------------------------------------------------

		} // not bypassed
	} // end channel loop

// --------------------- do not change this ---------------------
	totalSampleFrames += sampleFrames;
// --------------------------------------------------------------
}
#endif

//*******************************************************************************************************
//***************************************     END OF USER CODE     **************************************
//*******************************************************************************************************

//-----------------------------------------------------------------------------------------
bool CVSTTemplate::setBypass (bool onOff)
{
	if(onOff) 
		setParameter(kBypass,1.0f);
	else 
		setParameter(kBypass,0.0f);
	return false;
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::getParameterDisplay (VstInt32 index, char* text)
{
	if( (index >= PARAM_NUMBER) || (index < 0)) 
		return;

	float value = getParameter(index);
	ParamInfo param = params[index];
	switch (param.type) {
		case SHOW_IN_DB: dB2string (param.fValue, text, kVstMaxParamStrLen); break;
		case SHOW_IN_HZ: Hz2string (param.fValue, text, kVstMaxParamStrLen); break;
		case SHOW_IN_MS: ms2string (param.fValue, text, kVstMaxParamStrLen); break;
		case SHOW_IN_PC: int2string ((int)(param.fValue*100), text, kVstMaxParamStrLen); break;
		case SHOW_LOG:
		case SHOW_FLOAT: float2string (param.fValue, text, kVstMaxParamStrLen); break;
		case SHOW_INTEG: int2string ((int)param.fValue, text, kVstMaxParamStrLen); break;
		case SHOW_ONOFF: vst_strncpy (text, param.fValue?"On":"Off", kVstMaxParamStrLen); break;
		default: vst_strncpy (text, "", kVstMaxParamStrLen);
	}
}

//-----------------------------------------------------------------------------------------
bool CVSTTemplate::isParameterReadOnly (VstInt32 index)
{
	return params[index].bReadOnly;
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::setParameter (VstInt32 index, float value)
{

	if( (index >= PARAM_NUMBER) || (index <0 )) 
		return;

	ParamInfo *pParam = &params[index];
	if (pParam->bReadOnly)
		return;

	if ( fabs (lastVSTparameterValues[index] - value) < 0.00001 )
		return;
	
	// set real parameter values
	setRealValue (index, value);
	if( parameterChanged (index, value) )
		lastVSTparameterValues[index] = value;
	else
		setRealValue (index, lastVSTparameterValues[index]);

	// set editor parameters
#ifdef def_Editor
	if (editor)
		((AEffGUIEditor*)editor)->setParameter (index, value);
#endif
	
}

//-----------------------------------------------------------------------------------------
float CVSTTemplate::getParameter (VstInt32 index)
{
	if( (index >= PARAM_NUMBER) || (index < 0)) 
		return 0.0f;

	ParamInfo *pParam = &params[index];
	float value = 0.0f;
	switch (pParam->type) {
		case SHOW_LOG:
			value = log10f((pParam->fValue - pParam->fMin) / (pParam->fMax - pParam->fMin) * 9.0f + 1.0f);
			break;
		default: 
			value = (pParam->fValue - pParam->fMin) / (pParam->fMax - pParam->fMin);
	}
	return value;
}

//-----------------------------------------------------------------------------------------
float CVSTTemplate::getRealValue (VstInt32 index)
{
	ParamInfo *pParam = &params[index];
	return pParam->fValue;
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::setRealValue (VstInt32 index, float value)
{
	ParamInfo *pParam = &params[index];
	switch (pParam->type) {
		case SHOW_ONOFF:
			pParam->fValue = (value<0.5f)?0.0f:1.0f;
			break;
		case SHOW_INTEG:
			pParam->fValue = floor((pParam->fMin + value * (pParam->fMax - pParam->fMin) + 0.5f));
			break;
		case SHOW_LOG:
			pParam->fValue = pParam->fMin + (pow(10.0f,value) - 1.0f)/9.0f * (pParam->fMax - pParam->fMin);
			break;
		default: 
			pParam->fValue = pParam->fMin + value * (pParam->fMax - pParam->fMin);
	}
}

//-------------------------------------------------------------------------------------------------------
AudioEffect* createEffectInstance (audioMasterCallback audioMaster)
{
	return new CVSTTemplate (audioMaster);
}

//-----------------------------------------------------------------------------------------
bool CVSTTemplate::allocFloatTempBuffers (VstInt32 sampleFrames)
{
#ifdef def_tempInput
	// allocate temporary input buffer
	if(sampleFramesAllocated < sampleFrames)
		if(pfloatTempInput)
		{
			free(pfloatTempInput);
			pfloatTempInput = NULL;
			sampleFramesAllocated = 0;
		}

	if(!pfloatTempInput)
	{
		pfloatTempInput = (float*)calloc(sampleFrames, sizeof(float));
		if(pfloatTempInput)
			sampleFramesAllocated = sampleFrames;
		else 
			return false;
	}
#endif

#ifdef def_tempOutput
	// allocate temporary output buffer
	if(sampleFramesAllocated < sampleFrames)
		if(pfloatTempOutput)
		{
			free(pfloatTempOutput);
			pfloatTempOutput = NULL;
			sampleFramesAllocated = 0;
		}

	if(!pfloatTempOutput)
	{
		pfloatTempOutput = (float*)calloc(sampleFrames, sizeof(float));
		if(pfloatTempOutput)
			sampleFramesAllocated = sampleFrames;
		else 
			return false;
	}
#endif
	return true;
}

//-----------------------------------------------------------------------------------------
bool CVSTTemplate::allocDoubleTempBuffers (VstInt32 sampleFrames)
{
#ifdef def_tempInput
	// allocate temporary input buffer
	if(sampleFramesAllocated < sampleFrames)
		if(pdoubleTempInput)
		{
			free(pdoubleTempInput);
			pdoubleTempInput = NULL;
			sampleFramesAllocated = 0;
		}

	if(!pdoubleTempInput)
	{
		pdoubleTempInput = (double*)calloc(sampleFrames, sizeof(double));
		if(pdoubleTempInput)
			sampleFramesAllocated = sampleFrames;
		else 
			return false;
	}
#endif

#ifdef def_tempOutput
	// allocate temporary output buffer
	if(sampleFramesAllocated < sampleFrames)
		if(pdoubleTempOutput)
		{
			free(pdoubleTempOutput);
			pdoubleTempOutput = NULL;
			sampleFramesAllocated = 0;
		}

	if(!pdoubleTempOutput)
	{
		pdoubleTempOutput = (double*)calloc(sampleFrames, sizeof(double));
		if(pdoubleTempOutput)
			sampleFramesAllocated = sampleFrames;
		else 
			return false;
	}
#endif
	return true;
}

//-----------------------------------------------------------------------------------------
float* CVSTTemplate::getInputBuffer(float** inputs, VstInt32 sampleFrames, VstInt32 channel)
{
#ifdef def_tempInput
	memcpy(pfloatTempInput, inputs[channel], sampleFrames * sizeof(float));
	return pfloatTempInput;
#else
	return inputs[channel];
#endif
}

//-----------------------------------------------------------------------------------------
double* CVSTTemplate::getInputBuffer(double** inputs, VstInt32 sampleFrames, VstInt32 channel)
{
#ifdef def_tempInput
	memcpy(pdoubleTempInput, inputs[channel], sampleFrames * sizeof(double));
	return pdoubleTempInput;
#else
	return inputs[channel];
#endif
}

//-----------------------------------------------------------------------------------------
float* CVSTTemplate::getOutputBuffer(float** outputs, VstInt32 sampleFrames, VstInt32 channel)
{
#ifdef def_tempOutput
	memset(pfloatTempOutput, 0, sampleFrames * sizeof(float));
	return pfloatTempOutput;
#else
	return outputs[channel];
#endif
}

//-----------------------------------------------------------------------------------------
double* CVSTTemplate::getOutputBuffer(double** outputs, VstInt32 sampleFrames, VstInt32 channel)
{
#ifdef def_tempOutput
	memset(pdoubleTempOutput, 0, sampleFrames * sizeof(double));
	return pdoubleTempOutput;
#else
	return outputs[channel];
#endif
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::flushOutputBuffer(float** outputs, VstInt32 sampleFrames, VstInt32 channel)
{
#ifdef def_tempOutput
	memcpy(outputs[channel], pfloatTempOutput, sampleFrames * sizeof(float));
#endif
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::flushOutputBuffer(double** outputs, VstInt32 sampleFrames, VstInt32 channel)
{
#ifdef def_tempOutput
	memcpy(outputs[channel], pdoubleTempOutput, sampleFrames * sizeof(double));
#endif
}

//-------------------------------------------------------------------------------------------------------
CVSTTemplate::CVSTTemplate (audioMasterCallback audioMaster)
: AudioEffectX (audioMaster, 1, PARAM_NUMBER)
{	
	// sort parameters
	for (int i = 0; i < PARAM_NUMBER; i++)
		params[unsortedParams[i].tag] = unsortedParams[i];

	// VST init functions
	setNumInputs (def_ChNumber);		
	setNumOutputs (def_ChNumber);
	setUniqueID (def_UniqueID);
	canProcessReplacing ();

	editor = NULL;
	memset(&lastVSTparameterValues,0,sizeof(float)*PARAM_NUMBER);
#ifdef def_isSynth
	isSynth();
#endif;

#ifdef def_canDoubleReplacing
	canDoubleReplacing ();
#endif

	vst_strncpy (programName, def_DefaultProgName, kVstMaxProgNameLen);
	for ( int index=0; index<PARAM_NUMBER; index++ )
		params[index].fValue = params[index].fDefault;

#ifdef def_Editor
	editor = new CEditor (this);
#endif

	setUserVariables();

	// init VST parameters
	for(int index = 0; index < PARAM_NUMBER; index++)
		params[index].fValue = params[index].fDefault;

	for(int index = 0; index < PARAM_NUMBER; index++)
		parameterChanged(index, params[index].fDefault);

	pfloatTempInput  = NULL;
	pdoubleTempInput = NULL;
	pfloatTempOutput  = NULL;
	pdoubleTempOutput = NULL;
	sampleFramesAllocated = 0;
	totalSampleFrames = 0;
}

//-------------------------------------------------------------------------------------------------------
CVSTTemplate::~CVSTTemplate ()
{
	clearUserVariables();
	if(pfloatTempInput)
		free(pfloatTempInput);
	if(pdoubleTempInput)
		free(pdoubleTempInput);
	if(pfloatTempOutput)
		free(pfloatTempOutput);
	if(pdoubleTempOutput)
		free(pdoubleTempOutput);
}

//-------------------------------------------------------------------------------------------------------
unsigned long CVSTTemplate::getProcessedSamples (void)
{
	return totalSampleFrames;
}

//-------------------------------------------------------------------------------------------------------
double CVSTTemplate::getProcessingTime (void)
{
	return totalSampleFrames/(double)getSampleRate();
}

//-------------------------------------------------------------------------------------------------------
void CVSTTemplate::setProgramName (char* name)
{
	vst_strncpy (programName, name, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
VstInt32 CVSTTemplate::getProgram ()
{
	return 0;
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::getProgramName (char* name)
{
	vst_strncpy (name, programName, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::setProgram (VstInt32 program)
{

}

//-----------------------------------------------------------------------------------------
bool CVSTTemplate::getProgramNameIndexed (VstInt32 category, VstInt32 index, char *text)
{
	if(index==0)
	{
		vst_strncpy (text, programName, kVstMaxProgNameLen);
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::getParameterName (VstInt32 index, char* label)
{
	if (index<=PARAM_NUMBER) 
		vst_strncpy (label, params[index].cName, kVstMaxParamStrLen);
	else
		vst_strncpy (label, "", kVstMaxParamStrLen);
}

//-----------------------------------------------------------------------------------------
void CVSTTemplate::getParameterLabel (VstInt32 index, char* label)
{
	if (index<=PARAM_NUMBER) 
		vst_strncpy (label, params[index].cLabel, kVstMaxParamStrLen);
	else
		vst_strncpy (label, "", kVstMaxParamStrLen);
}

//------------------------------------------------------------------------
bool CVSTTemplate::getEffectName (char* name)
{
	vst_strncpy (name, def_EffectName, kVstMaxEffectNameLen);
	return true;
}

//------------------------------------------------------------------------
bool CVSTTemplate::getProductString (char* text)
{
	vst_strncpy (text, def_ProductString, kVstMaxProductStrLen);
	return true;
}

//------------------------------------------------------------------------
bool CVSTTemplate::getVendorString (char* text)
{
	vst_strncpy (text, def_VendorString, kVstMaxVendorStrLen);
	return true;
}

//-----------------------------------------------------------------------------------------
VstInt32 CVSTTemplate::getVendorVersion ()
{ 
	return def_VendorVersion;
}

//-----------------------------------------------------------------------------------------
