//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Version 2.4		$Date: 2006/11/13 09:08:27 $
//
// Category     : VST 2.x SDK Samples
// Filename     : vst_temp.h
// Created by   : Steinberg Media Technologies, Jiri Schimmel
// Description  : Plugin template
//
//  2006, Steinberg Media Technologies, All Rights Reserved
//
// Plug-In Template
// Version 1.2		$Date: 2008/05/03 $
// Created by   : Jiri Schimmel
// Description  : VST Plug-In Template
//
//  2008, Brno University of Technology
//-------------------------------------------------------------------------------------------------------

#ifndef __vsttemp__
#define __vsttemp__

#include <windows.h>

#include "../vst2.x/audioeffectx.h"
#include "vst_temp_defs.h"

#include "core.h"
#include "matica.h"


//-------------------------------------------------------------------------------------------------------

typedef struct sParamInfo
{
	int		tag;
	float	fValue;
	float	fDefault;
	float	fMin;
	float	fMax;
	char	cName[kVstMaxParamStrLen];
	char	cLabel[kVstMaxParamStrLen];
	char	type;
	bool	bReadOnly;
} ParamInfo;

enum {
	SHOW_VOID,
	SHOW_FLOAT,
	SHOW_IN_DB,
	SHOW_IN_HZ,
	SHOW_IN_MS,
	SHOW_IN_PC,
	SHOW_INTEG,
	SHOW_LOG,
	SHOW_ONOFF,
};

class CVSTTemplate : public AudioEffectX
{

public:
	CVSTTemplate (audioMasterCallback audioMaster);
	~CVSTTemplate ();

	// Processing
	virtual void processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames);
#ifdef def_canDoubleReplacing
	virtual void processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames);
#endif
	// Program
	virtual void setProgramName (char* name);
	virtual void getProgramName (char* name);
	virtual VstInt32 getProgram ();
	virtual bool getProgramNameIndexed (VstInt32 category, VstInt32 index, char *text);
	virtual void setProgram (VstInt32 program);

	// Parameters
	virtual void setParameter (VstInt32 index, float value);
	virtual float getParameter (VstInt32 index);
	virtual void getParameterLabel (VstInt32 index, char* label);
	virtual void getParameterDisplay (VstInt32 index, char* text);
	virtual void getParameterName (VstInt32 index, char* text);

	virtual bool getEffectName (char* name);
	virtual bool getVendorString (char* text);
	virtual bool getProductString (char* text);
	virtual VstInt32 getVendorVersion ();
	virtual bool setBypass (bool onOff);

	bool isParameterReadOnly (VstInt32 index);

protected:
	bool parameterChanged (VstInt32 index, float value);
	void setUserVariables (void);
	void clearUserVariables (void);

	void setRealValue (VstInt32 index, float value);
	float getRealValue (VstInt32 index);
	unsigned long getProcessedSamples (void);
	double getProcessingTime (void);

	char programName[kVstMaxProgNameLen + 1];
	float lastVSTparameterValues[PARAM_NUMBER];
	float  *pfloatTempInput, *pfloatTempOutput;
	double *pdoubleTempInput,*pdoubleTempOutput;
	VstInt32 sampleFramesAllocated;
	unsigned long totalSampleFrames;

	bool allocFloatTempBuffers  (VstInt32 sampleFrames);
	bool allocDoubleTempBuffers (VstInt32 sampleFrames);
	float*  getInputBuffer (float**  inputs,  VstInt32 sampleFrames, VstInt32 channel);
	double* getInputBuffer (double** inputs,  VstInt32 sampleFrames, VstInt32 channel);
	float*  getOutputBuffer(float**  outputs, VstInt32 sampleFrames, VstInt32 channel);
	double* getOutputBuffer(double** outputs, VstInt32 sampleFrames, VstInt32 channel);
	void  flushOutputBuffer(float**  outputs, VstInt32 sampleFrames, VstInt32 channel);
	void  flushOutputBuffer(double** outputs, VstInt32 sampleFrames, VstInt32 channel);

#include "user_variables.h"

};

#endif
